#pragma TextEncoding	= "UTF-8"
#pragma rtGlobals		= 3	
#pragma IgorVersion		= 6.36
#pragma ModuleName		= DifferenceGenerator
#pragma version			= 4.50
#pragma DefaultTab		= {3,20,4}		// set default tab width in Igor Pro 9 and later

static Constant kVersion = 4.50
static StrConstant kVersionDate = "03/2022"

// --------------------- Project Updater header ----------------------
// If you're using Igor Pro 8 or later and have Project Updater
// installed, this package can check periodically for new releases.
// https://www.wavemetrics.com/project/Updater
static Constant kProjectID = 21539 																// the project node on IgorExchange
static StrConstant ksShortTitle = "Difference Generator"										// the project short title on IgorExchange

//________________________________________________________________________________________________
//	Written by Stephan Thuermer - https://www.wavemetrics.com/user/chozo
//	Shift, scale, offset and compare data against reference data
//
//	Start the user interface via SpectraDifferenceGenerator(wave1,[wave2]). Wave 1 will be shifted and scaled,
//	and then compared against wave 2. Wave 2 is optional and can be selected later in the user interface.
//
//	It is possible to compare waves in user code using the function:
//		String result_message = WaveArithmeticKernel(WaveArithmeticStruct)
//	Here the structure WaveArithmeticStruct contains all the relevant parameters and wave references (see below).
//	Initialize the structure with WaveArithmeticInitialize() and then modify the desired parameters.
//	
// 2021-02-04 - ver. 4.20:	Initial public release.
// 2021-02-05 - ver. 4.30:	Added multiplication and division modes.
// 2021-03-07 - ver. 4.31:	Forgot to remove debug print command.
// 2021-03-17 - ver. 4.32:	Gauss convolution was not working well if data included NaNs.
//							Now the smoothing value is also displayed in the legend.
// 2021-03-19 - ver. 4.33:	Fixed an error message when loading specific old data sets.
// 2021-08-14 - ver. 4.40:	Improved edge behavior of the Gaussian smoothing (data edge are less likely to bulge).
//							Moved all controls into an external panel.
//							Added controls to directly set absolute X,Y, and scale values for changes.
//							Now Background Pinning honors smoothing as well.
// 2022-03-22 - ver. 4.50:	Fixed wrong panel size for windows.
//							Fixed bug: The arrows were reversed for some panel controls with negative delta scaling.
//							Fixed bug: Out-of-range error could happen for reversed (negative delta) waves.
//							Now multiple instances of the tool can be opened at the same time.
//							Added support for keyboard-shortcut (and other future) settings via external text file.
//							Improved backwards compatibility with older panels.
//							Comparisons can now be generated via user code by invoking WaveArithmeticKernel().
//							Axis split between comparison and result spectrum can now be dragged to a new position.
//							Added controls to for selecting and saving additional data generated from the result.
//							Added keyboard shortcuts for smoothing the input (home/end) and reference (alt+home/end) waves.
//________________________________________________________________________________________________

static Constant	kSaveAux = 0	// only for backwards compatibility
static Constant	kAuxMode = 0	// only for backwards compatibility
static StrConstant kModEnd	= "_mod"															// label for modified data (max. 4 characters for Igor version < 7)
static StrConstant kOutEnd	= "_dif;_sum;_mul;_div;"											// label for difference data
static StrConstant kAuxEnd	= "_smt;_int;_der;"													// label for additional data
static StrConstant kWorkingDir	= "Difference_Generator"										// name for the working folder inside Packages

// the following constants define the string keys for saving and loading parameters from wave notes
static StrConstant kScaleKey	= "Mod Scale"
static StrConstant kShiftKey	= "Mod Shift"
static StrConstant kOffsetKey	= "Mod Offset"
static StrConstant kModSmthKey	= "Mod Smooth"
static StrConstant kRefSmthKey	= "Ref Smooth"
static StrConstant kXCsrSetKey	= "Cursor Set"
static StrConstant kSetModeKey	= "Data difference;Data summed;Data multiplied;Data divided;"	// save key for all modes. The first one is not used for reloading, since this is the default mode.
static StrConstant kModeInvKey	= "Result inverted"
static StrConstant kPinBaseKey	= "Baseline scaled"
static StrConstant kRefNameKey	= "Reference name used"

static StrConstant kSettingsFileName = "Spectra Tools settings.dat"								// just for keyboard shortcuts for now.

Menu "Spectra Tools"
	DifferenceGenerator#MenuEntry(),/Q, CreateBrowser; DifferenceGenerator#LoadWrapper()
End

//################################################################################################

Structure WaveArithmeticStruct
	Variable mode			// mode of comparison: 1: Subtract (A-B) 2: Add (A+B) 3: Multiply (A*B) 4: Divide (A/B) => (optional; default is 1)
	Wave orgWave			// [mandatory] wave to compare 1 - will NOT be modified in any way
	Wave refWave			// [mandatory] wave to compare 2 - will NOT be modified in any way
	Wave cmpWave			// [mandatory] result of comparison between orgWave and refWave will be written here
	Wave modWave			// [optional] modified wave - saves the scaled and shifted version of orgWave
	Wave modRefWave			// [optional] modified reference wave - saves the smoothed version of refWave (if refSmooth > 0)
	Variable orgSmooth		// amount of smoothing for wave 1	(default 0)
	Variable refSmooth		// amount of smoothing for wave 2	(default 0)
	Variable baselineYval	// y value of the baseline			(default 0)
	Variable baselineXval	// x position of the baseline		(default NaN)
	Variable baselineFixed	// baseline is omitted from Y scaling if set to 1 (default 0)
	Variable invert			// invert the output in Y			(default 0)
	Variable xShift			// modify wave 1: x displacement (in wave's units)
	Variable yShift			// modify wave 1: y displacement (offset)
	Variable yScale			// modify wave 1: y scaling			(default 1)
	Variable GUIswitchPin	// GUI related value -> 1: pin baseline state was toggled
EndStructure

Function WaveArithmeticInitialize(s)
	STRUCT WaveArithmeticStruct &s
	s.mode			= 1
	s.orgSmooth		= 0
	s.refSmooth		= 0
	s.baselineYval	= 0
	s.baselineXval	= NaN
	s.baselineFixed	= 0
	s.invert		= 1
	s.xShift		= 0
	s.yShift		= 0
	s.yScale		= 1
	s.GUIswitchPin	= 0
	return 0
End

//################################################################################################

Function/S WaveArithmeticKernel(s)
	STRUCT WaveArithmeticStruct &s
	
	if (!WaveExists(s.orgWave) || !WaveExists(s.refWave) || !WaveExists(s.cmpWave))
		return "Input waves missing."
	endif
	if (WaveType(s.orgWave,1) != 1 || WaveDims(s.orgWave) > 1 || WaveType(s.refWave,1) != 1 || WaveDims(s.refWave) > 1 || WaveType(s.cmpWave,1) != 1 || WaveDims(s.cmpWave) > 1)		// make sure to only load 1D waves
		return "This tool works only for numeric 1D waves."
	endif
	if (s.mode < 1 || s.mode > 4 )
		return "Invalid mode "+num2str(s.mode)+". Choose between 1 and 4."
	endif

	if (!WaveExists(s.modWave))
		Duplicate/Free s.orgWave, s.modWave
	endif
	//++++++++++++++++++++++++++++++ modify input wave ++++++++++++++++++++++++++++++++++++++++++
	s.modWave = (s.orgWave - s.baselineYval*(s.baselineFixed==1))*s.yScale									// Y-scaling (with or without min. offset)
	SetScale/P x DimOffset(s.orgWave,0)+s.xShift, DimDelta(s.orgWave,0), WaveUnits(s.orgWave,0), s.modWave	// X-shift (scaling)
	Variable smthDrift = DiffGen_WaveConvolveGaussHelper(s.modWave, s.orgSmooth)				// smoothing (which can change the min wave value => baseline drift)
	s.yShift  += s.GUIswitchPin * (s.baselineYval*(1 - s.yScale) - smthDrift)					// if the pin-background state was changed: add or subtract the scaled amount to the offset (prevents 'snapping')
	s.modWave += s.yShift + (s.baselineYval	- smthDrift)*(s.baselineFixed==1)					// Y-shift (after smoothing)
	//+++++++++++++++++++++++++++++ smooth reference wave +++++++++++++++++++++++++++++++++++++++
	if (WaveExists(s.modRefWave))
		Duplicate/O s.refWave, s.modRefWave
		Wave refWave = s.modRefWave																// work with modified reference for the calculation
	else
		Duplicate/Free s.refWave tempRef
		Wave refWave = tempRef
	endif
	DiffGen_WaveConvolveGaussHelper(refWave, s.refSmooth)
	//+++++++++++++++++++++++++++++ now execute wave operation +++++++++++++++++++++++++++++++++++
	Variable X1 = pnt2x(s.modWave,0)
	Variable X2 = pnt2x(s.modWave,numpnts(s.modWave)-1)
	Variable mirrorO = sign(DimDelta(s.modWave,0)) == -1										// mirror for negative delta values => reverses assignment for X1 and X2
	Variable mirrorR = sign(DimDelta(s.refWave,0)) == -1
	SetScale/I x X1, X2, WaveUnits(s.cmpWave,0), s.cmpWave;	s.cmpWave = NaN						// first reset everything

	X1 = mirrorO ? pnt2x(s.modWave,numpnts(s.modWave)-1) : pnt2x(s.modWave,0)					// check the left side
	X2 = mirrorR ? pnt2x(s.refWave,numpnts(s.refWave)-1) : pnt2x(s.refWave,0)
	Variable Left = X1 > X2 ? X1 : X2															// the biggest value will be the left border
	X1 = mirrorO ? pnt2x(s.modWave,0) : pnt2x(s.modWave,numpnts(s.modWave)-1)					// check the right side
	X2 = mirrorR ? pnt2x(s.refWave,0) : pnt2x(s.refWave,numpnts(s.refWave)-1)
	Variable Right = X1 < X2 ? X1 : X2															// the smallest value will be the right border
	
	if (Right > Left)																			// look first if the waves are overlapping at all
		if (mirrorO && mirrorR)																	// reversed waves should get reversed differences
			SetScale/I x Right, Left, WaveUnits(s.cmpWave,0), s.cmpWave
		else
			SetScale/I x Left, Right, WaveUnits(s.cmpWave,0), s.cmpWave
		endif
		Switch (s.mode)																			// here the difference will be generated
			case 1:
				s.cmpWave  = s.modWave(x) - refWave(x)
			break
			case 2:
				s.cmpWave  = s.modWave(x) + refWave(x)
			break
			case 3:
				s.cmpWave  = s.modWave(x) * refWave(x)
			break
			case 4:
				s.cmpWave  = s.modWave(x) / refWave(x)
			break
		EndSwitch
		s.cmpWave *= s.invert ? -1 : 1
	endif
	//+++++++++++++++++++++++ update the wave notes : save parameters ++++++++++++++++++++++++++++
	String NoteText = "Difference created from:"
	NoteText += "\rReference Wave="	+ GetWavesDataFolder(s.refWave, 2)
	NoteText += "\rOriginal Wave="	+ GetWavesDataFolder(s.orgWave, 2)
	NoteText += "\rModification settings:"
	NoteText += "\r"+kShiftKey+"="	+num2str(s.xShift)
	NoteText += "\r"+kOffsetKey+"="	+num2str(s.yShift)
	NoteText += "\r"+kScaleKey+"="	+num2str(abs(s.yScale))
	NoteText += "\r"+kModSmthKey+"="+num2str(s.orgSmooth)
	
	if (numtype(s.baselineXval) == 0)
		NoteText += "\r"+kXCsrSetKey+"="+num2str(s.baselineXval)
	endif
	if (s.refSmooth > 0)
		NoteText += "\r"+kRefSmthKey+"="+num2str(s.refSmooth)
	endif
	if (s.mode >= 0)
		NoteText += "\r"+StringFromList(s.mode-1,kSetModeKey)
	endif
	if (s.invert)
		NoteText += "\r"+kModeInvKey
	endif
	if (s.baselineFixed == 0)
		NoteText += "\r"+kPinBaseKey
	endif
	
	String currNote = note(s.orgWave)
	Note/K s.cmpWave, NoteText
	if (strlen(currNote))
		NoteText[0] = currNote + "\r\r"
	endif
	Note/K s.modWave, NoteText
	
	return "Comparison successful."
End

//################################################################################################

static Function LoadWrapper()																	// calls DiffGen with the browser selection
	Wave/Z work = $GetBrowserSelection(0)	// first selection
	Wave/Z ref = $GetBrowserSelection(1)	// second selection
	if (WaveExists(work) == 0)
		return -1
	endif
	if (WaveExists(ref))
		SpectraDifferenceGenerator(work,refwave=ref)
	else
		SpectraDifferenceGenerator(work)
	endif
	return 0
End

//################################################################################################

static Function/S MenuEntry()
	Variable fileID
	String read = ""
	Open/Z/R fileID as ParseFilePath(1, FunctionPath(""), ":", 1, 0)+kSettingsFileName
	if (!V_flag)
		do
			FReadLine fileID, read
		while (strlen(read) > 0 && !StringMatch(read, "start difference generator*"))
		read = ReplaceString("\t", ReplaceString(" ", read, ""), "")
		read = StringByKey(ReplaceString(" ", "start difference generator", ""),read,"=","\r")
		Close fileID
	endif
	return "Compare and Modify Spectra ..."+SelectString(strlen(read),"","/"+read)
End

//################################################################################################

Function SpectraDifferenceGenerator(inwave,[refwave])
	Wave/Z inwave, refwave
	if (!WaveExists(inwave))
		return -1
	endif
	if (WaveType(inwave,1) != 1 || (WaveExists(refwave) && WaveType(refwave,1) != 1) || WaveDims(inwave) > 1 || (WaveExists(refwave) && WaveDims(refwave) > 1))
		Abort "This tool works only for numeric 1D waves."
	endif
	if (strlen(NameOfWave(inwave)) > 27 && IgorVersion() < 7.00)								// abort, if the wave name (plus the 4 character addition later) may exceed the 31 char limit (Igor 6)
		Abort "Wave name is too long ... 27 characters at maximum."
	endif
	//+++++++++++++++++++++++++++++++++ generate working variables ++++++++++++++++++++++++++++++
	String gTitle = "Spectra Arithmetic Tool (ver. "+num2str(kVersion)+" - "+kVersionDate+"): Modifying "
	String gName = UniqueName("DiffGenMainGraph", 6, 1), pName = "DiffGenPanel"
	String pPath = gName+"#"+pName
	String workF = "DF"+ReplaceString("DiffGenMainGraph",gName,"")+"_"+CleanupName(NameOfWave(inwave),0)[0,25]
	NewDataFolder/O root:Packages
	NewDataFolder/O root:Packages:$(kWorkingDir)
	NewDataFolder/O root:Packages:$(kWorkingDir):$(workF)										// the folder for temporary stuff
	DFREF stg = root:Packages:$(kWorkingDir):$(workF)
	
	String/G stg:locOriginal	= GetWavesDataFolder(inwave,2)
	String/G stg:locReference	= ""
	Make/D/O stg:Modifiers		= {0,0,1}														// contains temporary parameters: [0] = horizontal shift, [1] = vertical shift, [2] = multiplier
	//+++++++++++++++++++++++++++++++++++++ build GUI window +++++++++++++++++++++++++++++++++++++
	Variable TFTx	= str2num(StringFromList(3,StringByKey("SCREEN1",IgorInfo(0),":"),","))		// get the display resolution to align window better (center of full screen)
	Variable TFTy	= str2num(StringFromList(4,StringByKey("SCREEN1",IgorInfo(0),":"),","))
	Variable WinTop = 70, WinLeft = 200, pix = 72/ScreenResolution
	Variable WinBottom	= TFTy - 150
	Variable WinRight	= TFTx - 350
	
	Display/K=2/W=(WinLeft*pix, WinTop*pix, WinRight*pix, WinBottom*pix)/N=$gName as gTitle + NameOfWave(inwave)
	NewPanel/HOST=$gName/EXT=0/K=2/N=$pName/W=(0, 0, 250, 478) as "Spectra Modification"
	SetWindow $gName hook(DiffWindow)=DiffGen_ProcessWindowEvent								// hook function processes key events
	SetWindow $pPath hook(DiffWindow)=DiffGen_ProcessWindowEvent								// keeps track of window changes here as well (only button presses)
	SetWindow $pPath userdata(workFolder)=workF													// save work folder location
	SetWindow $gName userdata(procVersion)=num2str(kVersion)									// save procedure version
	//++++++++++++++++++++++++++++++ external panel with the controls +++++++++++++++++++++++++++++
	PopUpMenu ArithMode		,pos={10,8}		,size={175,23}	,title="Mode:"		,fstyle=1		,help={"Selects the current operation mode: A-B difference (- key), A+B addition (+ key), A*B multiplication (* key), A/B division (/ key)."}
	CheckBox InvertMode		,pos={195,10}	,size={50,23}	,title="\f04I\f00nvert"				,help={"Inverts the difference spectrum. (key: 'i')"}
	
	PopUpMenu OrigSelect	,pos={10,35}	,size={200,23}	,title="Modifying:"					,help={"Select the wave which will be edited."}
	PopUpMenu RefSelect		,pos={10,60}	,size={200,23}	,title="Reference:"					,help={"Select the reference wave to compare against."}
	Button bProgSwitch		,pos={220,35}	,size={22,44}	,title="]"			,fsize=20		,help={"Switches editing and reference waves."}
	
	GroupBox ConvLabel		,pos={8,90}		,size={235,45}	,title="Smoothing (Gauss convolution):"	,frame=0
	SetVariable WaveConv	,pos={20,110}	,size={105,23}	,title="Mod.:"						,help={"Convolves the modified wave with a Gaussian."}
	SetVariable RefWaveConv	,pos={130,110}	,size={105,23}	,title="Ref.:"						,help={"Convolves the reference wave with a Gaussian. Modification will not be saved."}

	GroupBox RelChangeLabel	,pos={8,140}	,size={235,133}	,title="Relative changes:"				,frame=0
	SetVariable Shift		,pos={15,162}	,size={130,23}	,title="delta X:"					,help={"The delta value by which the horizontal position will be shifted every step."}
	SetVariable Offset		,pos={15,191}	,size={130,23}	,title="delta Y:"					,help={"The delta value by which the vertical position will be shifted every step."}
	Button bUp				,pos={185,160}	,size={22,22}	,title="^"							,help={"Add a constant value to the data every step. Hold 'shift' ('ctrl/command') to accelerate (decelerate) the movement. Very fine movement with both ctrl/command + shift. (key: up arrow)"}
	Button bDown			,pos={185,188}	,size={22,22}	,title="v"							,help={"Remove a constant value from the data every step. Hold 'shift' ('ctrl/command') to accelerate (decelerate) the movement. Very fine movement with both ctrl/command + shift. (key: down arrow)"}
	Button bLeft			,pos={155,188}	,size={22,22}	,title="<"							,help={"Shift the spectrum to lower x values. Hold 'shift' ('ctrl/command') to accelerate (decelerate) the movement. Very fine movement with both ctrl/command + shift. (key: left arrow)"}
	Button bRight			,pos={213,188}	,size={22,22}	,title=">"							,help={"Shift the spectrum to higher x values. Hold 'shift' ('ctrl/command') to accelerate (decelerate) the movement. Very fine movement with both ctrl/command + shift. (key: right arrow)"}
	
	SetVariable Multiplier	,pos={15,220}	,size={130,23}	,title="Scale by:"					,help={"The intensity will be multiplied by this value every step."}			
	CheckBox PinBase		,pos={30,247}	,size={80,23}	,title="\f04P\f00in down Baseline:"	,help={"The baseline (wave minimum) stays fixed and will not be scaled. The fix point can be selected by setting cursor A or B on the reference wave (WaveR). (key: 'p')"}
	Button bStretch			,pos={155,220}	,size={80,20}	,title="Stretch"					,help={"Multiply intensity by the set percentage. Hold 'shift' ('ctrl/command') to accelerate (decelerate) the stretching. Very fine movement with both ctrl/command + shift. (key: page up)"}
	Button bShrink			,pos={155,245}	,size={80,20}	,title="Shrink"						,help={"Downsize intensity by the set percentage. Hold 'shift' ('ctrl/command') to accelerate (decelerate) the shrinking. Very fine movement with both ctrl/command + shift. (key: page down)"}
	
	GroupBox AbsChangeLabel	,pos={8,277}	,size={235,103}	,title="Absolute changes:"				,frame=0
	SetVariable AbsXshift	,pos={15,298}	,size={130,23}	,title="X:"							,help={"Set the absolute horizontal shift."}
	SetVariable AbsYshift	,pos={15,326}	,size={130,23}	,title="Y:"							,help={"Set the absolute vertical shift."}
	SetVariable AbsScale	,pos={15,354}	,size={130,23}	,title="Scale:"						,help={"Set the absolute scale."}

	Button bUndo			,pos={155,298}	,size={80,20}	,title="\f05R\f01eset:\f00 All"		,help={"Reset all modifications. Hold 'ctrl/command' to additionally unload the reference wave. (key: 'r')"}
	Button bUndoShift		,pos={155,325}	,size={35,20}	,title="X"							,help={"Reset horizontal shift."}
	Button bUndoOffset		,pos={200,325}	,size={35,20}	,title="Y"							,help={"Reset vertical shift."}
	Button bUndoScale		,pos={155,352}	,size={80,20}	,title="Scale"						,help={"Reset scaling."}
	
	Button bProgUndoQuit	,pos={10,390}	,size={60,44}	,title="Discard\r& Quit"			,help={"Discard changes and quit."}
	Button bProgSaveQuit	,pos={80,390}	,size={55,44}	,title="\f01Save\r& Quit"			,help={"Save changes and quit. Hold 'ctrl/command' key to just save the result without quitting."}	,valueColor=(65535,0,0)
	CheckBox FinishGraph	,pos={145,392}	,size={95,20}	,title="Display Result"				,help={"Display results graph at the end."}
	CheckBox AltSaveOption	,pos={145,418}	,size={95,20}	,title="Use Ref. Name"				,help={"Saves the difference wave with the name of the reference wave instead of the modified wave."}
	
	PopUpMenu popAuxMode	,pos={10,445}	,size={125,23}	,title="Show:"	,fstyle=1			,help={"Selects which type of additional data to show."}
	CheckBox cAuxSave		,pos={145,448}	,size={95,20}	,title="Save Aux. Data"				,help={"Also saves the additional data (green curve)."}
	
	if (CmpStr(IgorInfo(2), "Macintosh") == 0)
		ModifyControlList ControlNameList(pPath,";","*") fsize=10
	endif
	
	String modeList = "\"Subtract (A-B);Add (A+B);Multiply (A*B);Divide (A/B);\""
	String auxModeList = "\"Smoothed;Integrated;Derivative;\""
	
	ModifyControlList ControlNameList(pPath, ";", "b*")	,proc=DiffGen_ButtonControl						// define the procedure for modify buttons
	PopUpMenu OrigSelect	,bodywidth=140	,value=DiffGen_CreateWaveList("Orig")	,proc=DiffGen_PopupWaveSelect
	PopUpMenu RefSelect		,bodywidth=140	,value=DiffGen_CreateWaveList("Ref")	,proc=DiffGen_PopupWaveSelect
	PopUpMenu ArithMode		,bodywidth=115	,value=#modeList						,proc=DiffGen_ModeSelect
	PopUpMenu popAuxMode	,bodywidth=85	,value=#auxModeList						,proc=DiffGen_ModeSelect
	
	SetVariable Offset		,bodyWidth=80																// no limits and values for these controls (will be set in DiffGen_LoadWavesAndSettings())
	SetVariable Shift		,bodyWidth=80
	SetVariable Multiplier	,bodyWidth=80	,value=_NUM:1				,limits={0.01,inf,0.5}		,format="%g %"
	SetVariable AbsXshift	,bodyWidth=110	,value=stg:Modifiers[0]		,proc=DiffGen_WaveConvolve		// for the Abs controls procedure only updates the calculation
	SetVariable AbsYshift	,bodyWidth=110	,value=stg:Modifiers[1]		,proc=DiffGen_WaveConvolve	,format="%g"
	SetVariable AbsScale	,bodyWidth=90	,value=stg:Modifiers[2]		,proc=DiffGen_WaveConvolve	,format="%g x"
	SetVariable WaveConv	,bodyWidth=75								,proc=DiffGen_WaveConvolve
	SetVariable RefWaveConv	,bodyWidth=75	,value=_NUM:0				,proc=DiffGen_WaveConvolve
	CheckBox InvertMode													,proc=DiffGen_InvertMode
	CheckBox PinBase		,side=1			,value=1					,proc=DiffGen_PinBaseMode
	CheckBox FinishGraph					,value=1
	//++++++++++++++++++++++++ add the traces after the graph was built ++++++++++++++++++++++++++++	
	DoWindow/F $gName
	String recreate = DiffGen_LoadWavesAndSettings(pPath)										// create duplicates of the original waves and reload parameters if available (needs to be executed AFTER window controls were built)
	Wave/SDFR=stg WaveO, WaveR, WaveD, WaveA													// Info: WaveO = wave to edit; WaveR = reference wave; WaveD = difference wave; WaveA = auxiliary wave
	
	AppendToGraph WaveO
	AppendToGraph/C=(0,0,0)/L=Ldiff WaveD
	AppendToGraph/C=(0,40000,20000)/L=Ldiff WaveA
	ModifyGraph axisEnab(left)={0.31,1}, axisEnab(Ldiff)={0,0.29}, freePos(Ldiff)=0				// result is displayed at the bottom 30% of the graph
	ModifyGraph zero(Ldiff)=3, mirror=1, minor=1, standoff=0, ZisZ=1, notation(left)=1, gfSize=10
	SetAxis/A=2 Ldiff
	SetAxis/A=2 left
	Legend/C/N=GraphInfoBox/J/F=0/A=LT/X=2.00/Y=2.00 ""											// create trace legend
	Legend/C/N=ModificationInfoBox/J/F=0/A=RT/X=2.00/Y=2.00/B=1 ""								// create legend for modification info
	DoUpdate
	DiffGen_UpdateGraphLegends(pPath)															// create first legend entries
	
	WaveR = NaN
	if(WaveExists(refwave))																		// if reference wave is provided in function call
		DFREF waveDF = GetWavesDataFolderDFR(refwave)
		DFREF saveDF = GetDataFolderDFR()
		SetDataFolder waveDF
			PopUpMenu RefSelect ,mode=1 ,popvalue=NameOfWave(refwave)							// select reference wave
			DiffGen_WaveSelect(pPath, "RefSelect",NameOfWave(refwave))							// load reference wave
		SetDataFolder saveDF
	else																						// instead sets the old reference wave if available
		DiffGen_ReloadRefWaveSettings(pPath, recreate)											// reloads previous reference wave and sets relevant controls
	endif
	
	DiffGen_CalculateDifference(pPath)
	return 0
End
//++++++++++++++++++++++++++++++++++++++++ number rounding +++++++++++++++++++++++++++++++++++++++
static Function ChangePrecision(value, numSigDigits)											// round values to given number of digits
	variable value, numSigDigits
	string str
	sprintf str, "%.*g\r", numSigDigits, value
	return str2num(str)
End
//################################################################################################
static Function/DF getWorkFolder(pPath)															// grab current working folder from panel info
	String pPath
	String folder = ""
	if (strlen(pPath))
		folder = GetUserData(pPath,"","workFolder")
	endif
	if (strlen(folder))
		return root:Packages:$(kWorkingDir):$(folder)
	else
		return root:Packages:$(kWorkingDir)														// backwards compatibility with older sessions
	endif
End
//################################################################################################
static Function/S getPanel(gName)
	String gName
	String subPanel = StringFromList(0,RemoveFromList("AskQuitPanel",ChildWindowList(gName)))	// grab attached panel from list
	if(strlen(subPanel))
		return gName+"#"+subPanel
	else																						// compatibility with older versions
		return "DiffGenGraphMainWindow"
	endif
End
//################################################################################################
static Function/S getGraph(pPath)
	String pPath
	return StringFromList(0,pPath,"#")
End
//################################################################################################
static Function DiffGen_CalculateDifference(pPath, [undoPinning])
	String pPath
	Variable undoPinning																		// if pin background was toggled
	
	STRUCT WaveArithmeticStruct s
	WaveArithmeticInitialize(s)

	DFREF stg = getWorkFolder(pPath)
	SVar/SDFR=stg locOriginal, locReference
	Wave/SDFR=stg WaveO, WaveR, WaveD, WaveA, Modifiers											// Modifiers: [0] = horizontal shift, [1] = vertical shift, [2] = multiplier
	if (Exists(locReference))
		Wave s.refWave = $locReference
	endif
	Wave s.orgWave = $locOriginal
	Wave s.cmpWave = WaveD
	Wave s.modWave = WaveO
	Wave s.modRefWave = WaveR
	
	s.GUIswitchPin = ParamIsDefault(undoPinning) ? 0 : undoPinning								// this will offset the background in a way which keeps the apparent spectrum position constant
	ControlInfo/W=$pPath PinBase;		s.baselineFixed = V_Value
	ControlInfo/W=$pPath WaveConv;		s.orgSmooth = V_Value
	ControlInfo/W=$pPath RefWaveConv;	s.refSmooth = V_Value
	ControlInfo/W=$pPath InvertMode;	s.invert = V_Value										// normal or inverted
	ControlInfo/W=$pPath ArithMode;		s.mode = abs(V_flag) == 3 ? V_value : 1					// backwards compatibility => default is 1
	ControlInfo/W=$pPath SumMode;		s.mode = V_flag == 2 ? V_Value+1 : s.mode				// backwards compatibility => old checkbox
	//++++++++++++++++++++++++++++++++++++ modify the wave +++++++++++++++++++++++++++++++++++++++
	Variable yMin, csrPos
	DiffGen_FindBaseline(pPath, yMin, csrPos)													// use either the wave minimum or cursor location
	s.baselineYval = yMin
	s.baselineXval = csrPos
	s.xShift = numtype(Modifiers[0]) != 0 ? 0 : Modifiers[0]									// first, make sure these are proper numbers
	s.yShift = numtype(Modifiers[1]) != 0 ? 0 : Modifiers[1]
	s.yScale = numtype(Modifiers[2]) != 0 ? 1 : Modifiers[2]

	WaveArithmeticKernel(s)																		// actually do the comparison
	
	Modifiers[0] = s.xShift																		// write back changes to the parameters
	Modifiers[1] = s.yShift
	Modifiers[2] = s.yScale
	
	ControlInfo/W=$pPath popAuxMode
	Variable auxMode = abs(V_Flag) == 3 ? V_Value-1 : kAuxMode									// control added in version 4.5
	
	SetScale/P x DimOffset(s.cmpWave,0), DimDelta(s.cmpWave,0), WaveUnits(s.cmpWave,0), WaveA
	WaveA = s.cmpWave																			// Do something with the auxiliary wave
	Switch(auxMode)
		case 0:																					// smoothing
			Smooth round(DimSize(s.cmpWave,0)*0.05), WaveA
			ModifyGraph/Z/W=$getGraph(pPath) muloffset(WaveA)={0,0}
		break
		case 1:
			Integrate s.cmpWave /D=WaveA														// integration
			Variable Scale = abs(Wavemin(s.cmpWave)) < abs(Wavemax(s.cmpWave)) ? Wavemax(s.cmpWave)/Wavemax(WaveA) : Wavemin(s.cmpWave)/Wavemin(WaveA)
			ModifyGraph/Z/W=$getGraph(pPath) muloffset(WaveA)={0,Scale}
		break
		case 2:
			Differentiate s.cmpWave /D=WaveA													// derivative
			ModifyGraph/Z/W=$getGraph(pPath) muloffset(WaveA)={0,0}
		break
	EndSwitch
	Note/K WaveA, note(s.cmpWave)

	DiffGen_UpdateGraphLegends(pPath)
	return 0
End
//+++++++++++++++++++++++++++++++++ which baseline value to use ++++++++++++++++++++++++++++++++++
static Function DiffGen_FindBaseline(pPath, yMin, csrPos)
	String pPath
	Variable &yMin, &csrPos
	String gName = getGraph(pPath)
	
	DFREF stg = getWorkFolder(pPath)
	SVar/SDFR=stg locOriginal, locReference
	Wave origW = $locOriginal
	
	Variable csrAval = inf, csrBval = inf, csrAx = 0, csrBx = 0, useCsr = 0						// cursors may be used to set the baseline point
	if (StringMatch(CsrWave(A, gName),"WaveR"))
		csrAval	= vcsr(A, gName)
		csrAx	= xcsr(A, gName)
		useCsr	= 1
	endif
	if (StringMatch(CsrWave(B, gName),"WaveR"))
		csrBval	= vcsr(B, gName)
		csrBx	= xcsr(B, gName)
		useCsr	= 1
	endif
	if (useCsr)																					// save cursor location for recreation
		csrPos = csrAval < csrBval ? csrAx : csrBx
	else 
		csrPos = NaN
	endif
	
	yMin = useCsr == 0 ? Wavemin(origW) : min(csrAval,csrBval)									// use either the wave minimum or cursor location
	return 0 
End
//################################################################################################
Function DiffGen_ProcessWindowEvent(s)
	STRUCT WMWinHookStruct &s
	
	String gName = getGraph(s.winName)															// can be called from AskQuitPanel as well => construct panel path manually
	String pPath = getPanel(gName)

	if (s.EventCode > 2 && s.EventCode < 6 && CmpStr(gName,s.winName) == 0)						// for mouse events
		Variable axDivider, dragAxis = 0														// for a dynamic divider line
		sscanf StringByKey("axisEnab(x)", AxisInfo(gName,"left"), "="), "{%f,", axDivider
		axDivider -= 0.01																		// center split position
		GetWindow $gName, psizeDC
		Variable mouseClose = abs((s.mouseloc.v-v_top)-(1-axDivider)*(v_bottom-v_top)) < 5
		Variable rMousePos	= 1-(s.mouseloc.v-v_top)/(v_bottom-v_top)							// relative mouse position on the plot
	endif

	Variable HookTakeover = 0
	Switch (s.EventCode)
		case 17:	// kill request
			if (WinType(s.winName) == 1)														// comes from the main graph (there should not be a kill event from a panel anyway)
				HookTakeover = 1
				DFREF stg = getWorkFolder(pPath)
				SVar/SDFR=stg locOriginal
				Wave/SDFR=stg WaveO
				Wave origwave  = $(locOriginal)
				Wave/Z modwave = $(ParseFilePath(1, locOriginal, ":", 1, 0) + PossiblyQuoteName(NameOfWave(origwave) + kModEnd))
				
				Variable changed = 0
				if (WaveExists(modwave))
					changed = !EqualWaves(WaveO, modwave, 5)
				else
					changed = !EqualWaves(WaveO, origwave, 5)
				endif
				
				if (changed)
					if (!Wintype("#AskQuitPanel"))												// this will create a little panel in case of changes
						Variable pixConv = (CmpStr(IgorInfo(2), "Windows") == 0 && ScreenResolution == 96)? 72/ScreenResolution : 1
						Variable xCenter = (s.WinRect.right - s.WinRect.left)/2*72/ScreenResolution, yCenter = (s.WinRect.bottom - s.WinRect.top)/2*72/ScreenResolution
						NewPanel/N=AskQuitPanel/HOST=$s.WinName/W=(xCenter-105*pixConv,yCenter-38*pixConv,xCenter+105*pixConv,yCenter+38*pixConv)
						DrawText 20,28,"Quiting: Save made changes?"
						Button bProgSaveQuit	,pos={20,33}	,size={50,30}	,title="\f01Save"	,proc=DiffGen_ButtonControl	,help={"Save changes and quit. Hold the 'alt/option' key to save the difference spectrum with the name of the reference wave instead of the editing wave."}
						Button bProgUndoQuit	,pos={80,33}	,size={50,30}	,title="Discard"	,proc=DiffGen_ButtonControl	,help={"Discard changes and quit."}
						Button bProgCancel		,pos={140,33}	,size={50,30}	,title="Cancel"		,proc=DiffGen_ButtonControl
						SetActiveSubwindow $s.WinName											// restore focus to main window
					endif
				else
					DiffGen_ProgramExecute(pPath, "bProgUndoQuit")
				endif
			endif
		break
		case 11:	// keys
			HookTakeover = 1
			String Key = "", Cursors = ""
			Variable Ignore = 0																	// ignore = 1 will give Igor the control over this event
			Switch(s.keycode)																	// identify the pressed key
				case 11:
					Key = "bStretch"
					break
				case 12:
					Key = "bShrink"
					break
				case 28:
					Key = "bLeft"
					Ignore = GetKeyState(0) & 2^1												// move cursors with alt key
					break
				case 29:
					Key = "bRight"
					Ignore = GetKeyState(0) & 2^1
					break
				case 30:
					Key = "bUp"
					break
				case 31:
					Key = "bDown"
					break
				case 42:	// * key
					Key = "setMode_3"
					break
				case 43:	// + key
					Key = "setMode_2"
					break
				case 45:	// - key
					Key = "setMode_1"
					break
				case 47:	// / key
					Key = "setMode_4"
					break
				case 105:	// i key
					ControlInfo/W=$pPath InvertMode
					CheckBox InvertMode	,win=$pPath ,value=1-V_Value	
					break
				case 112:	// p key
					Key = "cPin"
					break	
				case 114:	// r key
					Key = "bUndo"
					break
				case 1:	// home key
					Key = "smthInc"
					break	
				case 4:	// end key
					Key = "smthDec"
					break
			EndSwitch

			Cursors = CsrInfo(A,gName)															// check, if one of the cursors is active: if yes give control back to Igor for cursor movement
			if (strlen(Cursors) && strsearch(Cursors,"/A=0",0,2)  == -1 && Ignore)
				HookTakeover = 0
				break
			endif
			Cursors = CsrInfo(B,gName)
			if (strlen(Cursors) && strsearch(Cursors,"/A=0",0,2)  == -1 && Ignore)
				HookTakeover = 0
				break
			endif
			DiffGen_ProcessEvents(pPath, Key)													// do the modification according to the pressed key
		break
				case 3: 	// mousedown
			if (mouseClose)																		// initialize axis drag state
				SetWindow $pPath userdata(axdragOn)="1"
				HookTakeover = 1
			endif
		break
		case 5: 	// mouseup
			SetWindow $pPath userdata(axdragOn)="0"												// release axis drag
		break
		case 4:		// mouse movement
			//++++++++++++++++++++++++++++++++ axis quick drag ++++++++++++++++++++++++++++++++++
			if (!WinType(pPath))																// catches slow window buildup in Igor 6
				break
			endif
			dragAxis = str2num(GetUserData(pPath, "", "axdragOn"))
			dragAxis = numtype(dragAxis) != 0 ? 0 : dragAxis
			if (dragAxis)
                GetMouse/W=$gName
                if (!(V_flag & 1))																// mouse button released? => maybe we missed a mouse-up event?
                    SetWindow $pPath userdata(axdragOn)="0"
                    dragAxis = 0
                endif
            endif
			DrawAction/W=$gName getgroup=axisSplitLine, delete									// draws and sets divider line for dynamic axis drag
			if (dragAxis)
				axDivider = limit(rMousePos, 0.15, 0.85)										// set new divider
				ModifyGraph/W=$gName axisEnab(left)	 = {axDivider + 0.01, 1}
				ModifyGraph/W=$gName axisEnab(Ldiff) = {0, axDivider - 0.01}
				HookTakeover = 1
			endif
			if (dragAxis || mouseClose)															// draw a line for visualization
				SetDrawEnv/W=$gName linefgc=(36000,15000,59000) ,dash=2 ,linethick= 1.50
				SetDrawEnv/W=$gName gstart, gname=axisSplitLine
				DrawLine/W=$gName -0.1, 1-axDivider, 1.1, 1-axDivider
				SetDrawEnv/W=$gName gstop, gname=axisSplitLine
				s.cursorCode = 6																// change mouse cursor
				s.doSetCursor = 1
			endif
		break
	EndSwitch
	return HookTakeover
End
//################################################################################################
static Function/S DiffGen_LoadWavesAndSettings(pPath)											// creates or copies the working waves from the originals
	String pPath
	DFREF stg = getWorkFolder(pPath)
	SVar/SDFR=stg locOriginal
	Wave/SDFR=stg Modifiers																		// Modifiers: [0] = horizontal shift, [1] = vertical shift, [2] = multiplier
	String origname	= ReplaceString("'",ParseFilePath(0, locOriginal, ":", 1, 0),"")
	Wave origwave	= $(locOriginal)
	Wave/Z modwave	= $(ParseFilePath(1, locOriginal, ":", 1, 0) + PossiblyQuoteName(origname + kModEnd))
	
	String recreate = "", outnotes = ""
	if (WaveExists(modwave))																	// duplicate pre-existing mod wave into temp folder
		Duplicate/O modwave stg:WaveO
		recreate = note(modwave)																// extract modify parameters from wave note
	else
		Duplicate/O origwave stg:WaveO
		Modifiers = 0; Modifiers[2] = 1															// reset modify parameters
	endif
	
	Duplicate/O stg:WaveO stg:WaveD, stg:WaveA													// duplicate original to difference and aux waves
	Wave/Z/SDFR=stg WaveO, WaveR, WaveD, WaveA
	WaveD = NaN; WaveA = NaN																	// set to NaNs at first to render the wave invisible
	if (!WaveExists(WaveR))
		Duplicate WaveO stg:WaveR																// create a dummy reference wave
		Wave/SDFR=stg WaveR
		WaveR = NaN
	endif
	
	Variable i
	for (i = 0; i < 4; i += 1)
		Wave/Z outwave	= $(ParseFilePath(1, locOriginal, ":", 1, 0) + PossiblyQuoteName(origname + StringFromList(i,kOutEnd)))
		if (WaveExists(outwave))
			outnotes = note(outwave)
			// *********************** Compatibility code: load old diff waves *******************
			Variable oldDiff = StringMatch(outnotes,"*Difference created from*") + StringMatch(outnotes,"*difference spectrum to*")
			// ***********************************************************************************
			if (!WaveExists(modwave) || oldDiff)												// if no modification wave exits try to load settings from outnote
				recreate = outnotes
			elseif (StringMatch(outnotes,"*"+kRefNameKey+"*"))									// the diff wave was saved with the reference name => switch edit and ref wave
				recreate += "/r"+kRefNameKey
			endif
			break
		endif
	endfor
	// ++++++++++++++++++++++ load saved control settings from wave note +++++++++++++++++++++++++
	Variable origsmth = 0, refsmth = 0, from = 0
	if (strlen(recreate))
		from = strsearch(recreate,"Difference created from:",0)
		from = from == -1 ? 0 : from
		recreate = recreate[from,inf]
		Modifiers[0] = str2num(StringByKey(kShiftKey, recreate, "=", "\r"))
		Modifiers[1] = str2num(StringByKey(kOffsetKey,recreate, "=", "\r"))
		Modifiers[2] = str2num(StringByKey(kScaleKey, recreate, "=", "\r"))
		origsmth = str2num(StringByKey(kModSmthKey,	recreate, "=", "\r"))
		refsmth	 = str2num(StringByKey(kRefSmthKey,	recreate, "=", "\r"))
		origsmth = numtype(origsmth) == 0 ? origsmth : 0
		refsmth  = numtype(refsmth) == 0 ? refsmth : 0
		// ************* Compatibility code: extract mod settings from wave directly *************
		if (numtype(sum(Modifiers)) != 0 && WaveExists(modwave))
			Modifiers[0] = ChangePrecision(DimOffset(modwave,0) - DimOffset(origwave,0),5)
			Modifiers[1] = ChangePrecision(Wavemin(modwave) - Wavemin(origwave),5)
			Modifiers[2] = round(( (Wavemax(modwave) - Wavemin(modwave)) / (Wavemax(origwave) - Wavemin(origwave)))*100000 ) / 100000
		endif
		if (numtype(sum(Modifiers)) != 0)
			Modifiers = 0; Modifiers[2] = 1														// no useful modifiers found
		endif
		// ***************************************************************************************
	endif
	
	String Unit = WaveUnits(origwave,0)
	Variable initY = abs(ChangePrecision(Wavemax(origwave)*0.001,1))							// initial offset value (0.1% of max Y value)
	Variable initX = abs(ChangePrecision(DimDelta(origwave,0),2))								// round to 2 significant digits
	if (strlen(Unit))
		Unit[0] = " "
	endif
	SetVariable Offset		,win=$pPath ,value=_NUM:initY		,limits={initY/50,inf,initY/4}	,format="%g"
	SetVariable Shift		,win=$pPath ,value=_NUM:initX		,limits={initX/50,inf,initX/4}	,format="%g" + Unit
	SetVariable WaveConv	,win=$pPath ,value=_NUM:origsmth	,limits={0,inf,initX/4}			,format="%g" + Unit
	SetVariable RefWaveConv	,win=$pPath ,value=_NUM:refsmth		,limits={0,inf,initX/4}			,format="%g" + Unit
	ControlInfo/W=$pPath AbsXshift
	if (abs(V_flag) == 5)																		// for newer versions => 4.4
		SetVariable AbsXshift	,win=$pPath	,limits={-inf,inf,initX/4}	,format="%g" + Unit
		SetVariable AbsYshift	,win=$pPath	,limits={-inf,inf,initY/4}
		SetVariable AbsScale	,win=$pPath	,limits={-inf,inf,0.1}
	endif
	CheckBox InvertMode	 	,win=$pPath ,value= StringMatch(recreate,"*"+kModeInvKey+"*")
	CheckBox PinBase	 	,win=$pPath ,value=!StringMatch(recreate,"*"+kPinBaseKey+"*")
	Variable setMode = 1																		// difference mode is default
	setMode = StringMatch(recreate,"*"+StringFromList(1,kSetModeKey)+"*") ? 2 : setMode
	setMode = StringMatch(recreate,"*"+StringFromList(2,kSetModeKey)+"*") ? 3 : setMode
	setMode = StringMatch(recreate,"*"+StringFromList(3,kSetModeKey)+"*") ? 4 : setMode
	ControlInfo/W=$pPath ArithMode
	if (abs(V_flag) == 3)
		PopUpMenu ArithMode	 ,win=$pPath ,mode=setMode
	endif
	return recreate																				// copy over to DiffGen_ReloadRefWaveSettings()
End
//################################################################################################
static Function DiffGen_ReloadRefWaveSettings(pPath, recreate)
	String pPath, recreate
	DFREF stg = getWorkFolder(pPath)
	SVar/SDFR=stg locOriginal

	Variable switched = StringMatch(recreate,"*"+kRefNameKey+"*")								// the diff wave was saved with the reference name => switch edit and ref wave	
	Variable refsmth  = str2num(StringByKey(kRefSmthKey, recreate, "=", "\r"))
	Variable csrPos	  = str2num(StringByKey(kXCsrSetKey, recreate, "=", "\r")) 
	refsmth = numtype(refsmth) == 0 ? refsmth : 0
	CheckBox AltSaveOption	,win=$pPath ,value=switched											// activate ref. name save option in this case
	
	String refWave	= SelectString(switched, StringByKey("Reference Wave", recreate,"=","\r"), StringByKey("Original Wave", recreate,"=","\r"))	// if switched load the original wave into ref. and switch them over later
	String refDir	= ParseFilePath(1, refWave, ":", 1, 0)
	String refName	= ReplaceString("'",ParseFilePath(0, refWave, ":", 1, 0),"")
	String origName	= ReplaceString("'",ParseFilePath(0, locOriginal, ":", 1, 0),"")
	
	// ****************** Compatibility code: extract wave info from old files *******************
	if (StrLen(refDir) == 0 || !DataFolderExists(refDir))
		if (StringMatch(recreate,"*Difference created from*"))
			refDir	= StringByKey("Folder", recreate,":","\r")
			refName	= StringByKey("Wave", recreate,":","\r")
			if (StringMatch(origName,refName))													// don't to load the same wave as both reference and original
				refName = ""
			endif
		elseif(StringMatch(recreate,"*difference spectrum to*"))
			refDir	= GetDataFolder(1)
			refName	= recreate[StrLen("difference spectrum to "),inf]
			refName	= ReplaceString("'",refName, "")
		endif
	endif
	if (StrLen(refDir) == 0 || !DataFolderExists(refDir))										// still not found
		refDir	= GetDataFolder(1)
	endif
	// *******************************************************************************************
	
	DFREF saveDF = GetDataFolderDFR()
	SetDataFolder refDir
	if(Exists(refName) == 1)
		PopUpMenu RefSelect ,win=$pPath ,popmatch=refName										// select the reference wave
		DiffGen_WaveSelect(pPath, "RefSelect",refName)											// load the old reference wave
		if (switched)
			SwitchWaves(pPath)
		endif
		SetVariable RefWaveConv	,win=$pPath ,value=_NUM:refsmth									// set ref wave convolution control
		if (numtype(csrPos) == 0)
			Cursor/W=$getGraph(pPath) A WaveR csrPos
		endif
	endif
	SetDataFolder saveDF
	return 0
End
//################################################################################################
static Function DiffGen_UpdateGraphLegends(pPath)
	String pPath
	String gName = getGraph(pPath)
	DFREF stg = getWorkFolder(pPath)
	Wave/SDFR=stg WaveO, WaveR, WaveD, WaveA
	
	String GraphText = ""
	WaveStats/Q WaveR
	if (V_numNans < DimSize(WaveR,0))															// if nothing is in there, don't generate the legend
		Variable A = -inf,	B = inf
		if (strlen(CsrInfo(A, gName)) > 0 && strlen(CsrInfo(B, gName)) > 0)						// get cursor info if available
			A = xcsr(A, gName)
			B = xcsr(B, gName)
			sprintf GraphText, "Area between %.1f %s and %.1f %s\r", A, WaveUnits(WaveO,0), B, WaveUnits(WaveO,0)
		endif
		MatrixOP/Free tempO = replaceNaNs(WaveO,0) 												// makes sure there are no NaN values
		MatrixOP/Free tempR = replaceNaNs(WaveR,0)
		MatrixOP/Free tempD = replaceNaNs(WaveD,0)
		SetScale/P x, DimOffset(WaveO,0), DimDelta(WaveO,0), tempO
		SetScale/P x, DimOffset(WaveR,0), DimDelta(WaveR,0), tempR
		SetScale/P x, DimOffset(WaveD,0), DimDelta(WaveD,0), tempD
		Variable AreaO = Area(tempO,A,B), AreaR = Area(tempR,A,B), AreaD = Area(tempD,A,B)		// calculate each area
		GraphText += "\\s(WaveO) modified\t(area: "		+ num2str(AreaO)
		GraphText += ")\r\\s(WaveR) reference\t(area: "	+ num2str(AreaR) + "; ratio: " + num2str(abs(AreaR / AreaO))
		GraphText += ")\r\\s(WaveD) difference\t(area: "+ num2str(AreaD) + "; ratio: " + num2str(abs(AreaD / AreaO))
		
		ControlInfo/W=$pPath popAuxMode
		Variable auxMode = abs(V_Flag) == 3 ? V_Value-1 : kAuxMode								// control added in version 4.5
		Switch(auxMode)
			case 0:
				GraphText += ")\r\\s(WaveA) smoothed"
			break
			case 1:
				GraphText += ")\r\\s(WaveA) integrated (area: "	+ num2str(Area(WaveA,A,B)) + ")"
			break
			case 2:
				GraphText += ")\r\\s(WaveA) derivative (area: "	+ num2str(Area(WaveA,A,B)) + ")"
			break
		EndSwitch	
	endif
	Legend/C/N=GraphInfoBox/W=$gName/J GraphText												// update trace legend
	DiffGen_UpdateModLegend(stg, gName)															// update modification legend
	return 0
End
//################################################################################################
static Function DiffGen_UpdateModLegend(stg, gName)
	DFREF stg
	String gName
	Wave/SDFR=stg WaveO, Modifiers																// Modifiers: [0] = horizontal shift, [1] = vertical shift, [2] = multiplier
	SVar/SDFR=stg locOriginal
	Wave origwave = $locOriginal
	// ++++++++++++++++++++++++ right legend (modification details) ++++++++++++++++++++++++++++++++++++++++++++++
	String BoxText	= "", unit = WaveUnits(WaveO,0)
	Variable offset, smth
	if (abs(Modifiers[0]) > 10^-16)
		BoxText += num2str(Modifiers[0]) + " " + SelectString(strlen(unit),"units",unit) + " shift\r"
	endif
	offset = WaveMin(WaveO)-WaveMin(origwave)
	if (abs(offset) > 10^-10)
		BoxText += num2str(offset) + " offset\r"
	endif
	if (Modifiers[2] != 1)
		BoxText += num2str(abs(Modifiers[2])) + " x scaled\r"
	endif
	smth = str2num(StringByKey(kModSmthKey,note(WaveO), "=", "\r"))
	if (numtype(smth) == 0 && smth > DimDelta(WaveO,0)*10^-9)									// avoid rounding problems
		BoxText += num2str(smth) + SelectString(strlen(unit),""," ") + unit + " smoothed\r"
	endif
	smth = str2num(StringByKey(kRefSmthKey,note(WaveO), "=", "\r"))
	if (numtype(smth) == 0 && smth > DimDelta(WaveO,0)*10^-9)
		BoxText += num2str(smth) + SelectString(strlen(unit),""," ") + unit + " ref. smooth\r"
	endif
	BoxText = RemoveEnding(BoxText,"\r")
	BoxText[0] = "\K(65280,0,0)"
	Legend/C/N=ModificationInfoBox/W=$gName/J BoxText											// update legend for modification info
	return 0
End
//#################################support functions for control elements#########################
Function DiffGen_ModeUpdater(s) : CheckBoxControl												// only for backwards compatibility
	STRUCT WMCheckboxAction& s
	if(s.eventCode == 2)
		if (StringMatch(s.ctrlName,"PinBase"))
			DiffGen_CalculateDifference(s.win, undoPinning=(1-2*s.checked))
		else
			DiffGen_CalculateDifference(s.win)
		endif
	endif
	return 0
End
//################################################################################################
Function DiffGen_PinBaseMode(s) : CheckBoxControl
	STRUCT WMCheckboxAction& s
	if(s.eventCode == 2)
		DiffGen_CalculateDifference(s.win, undoPinning=(1-2*s.checked))							// undo the pinning state before doing the calculation
	endif
	return 0
End
//################################################################################################
Function DiffGen_InvertMode(s) : CheckBoxControl
	STRUCT WMCheckboxAction& s
	if(s.eventCode == 2)
		DiffGen_CalculateDifference(s.win)
	endif
	return 0
End
//################################################################################################
Function DiffGen_WaveConvolve(s) : SetVariableControl
	STRUCT WMSetVariableAction &s
	if (s.eventCode == 1 || s.eventCode == 2)
		DiffGen_CalculateDifference(s.win)
	endif
	return 0
End
//############################# convolve with a prepared Gaussian ################################
static Function DiffGen_WaveConvolveGaussHelper(in, width)
	Wave in
	Variable width
	if (width == 0 || numtype(width) != 0)
		return 0
	endif
	
	Variable pnt, minVal = WaveMin(in)
	Variable dx  = DimDelta(in,0), size = DimSize(in,0)
	width = abs(width/dx)/sqrt(2)																// width normalized by x scaling and convert to expected Gauss 'width'
	pnt = round(max(abs(10*width),11))															// create a wave 10 times the Gauss width to get the full Gauss in (5 sigma on each side)
	pnt = min(pnt, 2*numpnts(in)+1)																// doesn't have to be larger than two times the full input data
	pnt = pnt+!mod(pnt, 2)																		// force odd size

	Make/FREE/D/N=(pnt) GaussWave	
	GaussWave = Gauss(x, (pnt-1)/2, width)
	Variable A = sum(GaussWave)																	// make sure the Gauss area limited even for a few points
	if (A > 1)
		GaussWave /= A
	endif
	
	Variable NaNs = 0, num = NaN																// find NaNs at the start and end points
	do
		num = in[NaNs]
		NaNs += 1
	while (numtype(num) != 0 && NaNs < size-1)
	do
		num = in[size-1]
		size -= 1
	while (numtype(num) != 0 && size > 0)
	NaNs -= 1
	size += 1
	
	Make/FREE/D/N=(2*pnt+size-NaNs) temp = in[NaNs]												// enlarge the wave to get the full convolution range
	temp[pnt,size+pnt-1-NaNs] = in[p-pnt+NaNs]
	temp[size+pnt-NaNs,size+2*pnt-1-NaNs] = in[size-1]
	MatrixOP/Free temp = replaceNaNs(temp,0)													// makes sure there are no NaN values in-between
	
	Convolve/A GaussWave temp
	in[NaNs,size-1] = temp[p+pnt-NaNs]
	
	return WaveMin(in)-minVal																	// return by how much the baseline has shifted
End
//################################################################################################
Function DiffGen_ModeSelect(s) : PopupMenuControl
	STRUCT WMPopupAction &s
	if (s.eventCode == 2)
		DiffGen_CalculateDifference(s.win)
		ModifyControl bUndo ,win=$s.win ,activate												// move focus away from the popup (prevents accidental selection of a different wave with the cursor keys)
	endif
	return 0
End
//################################################################################################
Function DiffGen_PopupWaveSelect(s) : PopupMenuControl
	STRUCT WMPopupAction &s
	if (s.eventCode == 2 && (s.popNum > 1 || StringMatch(s.popStr,"none")))
		DiffGen_WaveSelect(s.win, s.ctrlName, s.popStr)
		ModifyControl bUndo ,win=$s.win ,activate												// move focus away from the popup (prevents accidental selection of a different wave with the cursor keys)
	endif
	return 0
End
//+++++++++++++++++++++++++++++++++++++ popup help function +++++++++++++++++++++++++++++++++++++
Function/S DiffGen_CreateWaveList(Type)															// help function to create the wave lists for the drop down menus
	String Type
	String topGraph = StringFromList(0,WinList("DiffGen*", ";","WIN:1"))						// have to assume the popup is invoked while the attached graph is on top
	String pPath = getPanel(topGraph)
	if (strlen(topGraph))
		DFREF stg = getWorkFolder(pPath)
		SVar/SDFR=stg locOriginal
		String name	 = ReplaceString("'",ParseFilePath(0, locOriginal, ":", 1, 0),"")
		String waves = WaveList("!" + name,";", "DIMS:1,TEXT:0")										// exclude original wave name (prevents creating difference with identical wave)
		waves = SortList(waves, ";", 16)
		ControlInfo/W=$pPath popAuxMode;	Variable auxMode = abs(V_Flag) == 3 ? V_Value-1 : kAuxMode	// control added in version 4.5
		ControlInfo/W=$pPath ArithMode;		Variable setMode = abs(V_flag) == 3 ? V_value-1 : 0
		waves = RemoveFromList(name+StringFromList(setMode,kOutEnd)+";"+name+kModEnd+";"+name+StringFromList(auxMode,kAuxEnd)+";",waves)	// remove the associated result waves (cannot remove reference name here or switching will not work)
		return SelectString(StringMatch(Type,"Ref"), name + ";", "none;") + waves						// in case of reference wave popup - create wave list from current folder without original wave
	else
		return ""
	endif
End
//################################################################################################
static Function DiffGen_WaveSelect(pPath, type, name)
	String pPath, type, name
	
	DFREF stg = getWorkFolder(pPath)
	SVar/SDFR=stg locOriginal, locReference
	
	String gName = getGraph(pPath)
	StrSwitch (type)
		case "OrigSelect":
			String oldName = ReplaceString("'",ParseFilePath(0, locOriginal, ":", 1, 0),"")		// previous wave name
			locOriginal = GetWavesDataFolder($name, 2)
			DiffGen_LoadWavesAndSettings(pPath)													// copy waves into working folder
			GetWindow  $gName, wtitle
			DoWindow/T $gName, ReplaceString(oldName,S_value,"") + name							// change the window title
		break
		case "RefSelect":
			Wave/SDFR=stg WaveR
			locReference = ""; WaveR = NaN														// reset previous reference
			RemoveFromGraph/W=$gName/Z WaveR
			if (!StringMatch(name,"none"))
				locReference = GetWavesDataFolder($name, 2)
				Duplicate/O $name stg:WaveR
				AppendToGraph/W=$gName/C=(0,0,65280) WaveR
			endif
		break
	endswitch
	DiffGen_CalculateDifference(pPath)
	return 0
End
//################################################################################################
static Function SwitchWaves(pPath)																// switches reference and edit wave selection
	String pPath
	DFREF stg = getWorkFolder(pPath)
	DFREF sav = GetDataFolderDFR()
	SVar/SDFR=stg locOriginal, locReference
	Wave OrigWave  = $locOriginal
	Wave/Z RefWave = $locReference
	if (WaveExists(RefWave))																	// switch waves by selecting the reference in the original's place and vice versa
		String OrigSelection = NameOfWave(OrigWave)
		String RefSelection  = NameOfWave(RefWave)
		
		SetDataFolder GetWavesDataFolderDFR(RefWave)
		PopUpMenu OrigSelect ,win=$pPath ,popMatch=RefSelection
		DiffGen_WaveSelect(pPath, "OrigSelect", RefSelection)
		
		SetDataFolder GetWavesDataFolderDFR(OrigWave)
		PopUpMenu RefSelect	 ,win=$pPath ,popMatch=OrigSelection
		DiffGen_WaveSelect(pPath, "RefSelect", OrigSelection)
	endif
	SetDataFolder sav
	return 0
End
//################################################################################################
Function DiffGen_ButtonControl(s) : ButtonControl
	STRUCT WMButtonAction &s
	if (s.eventCode == 2)
		if (StringMatch(s.ctrlName,"bProg*"))
			DiffGen_ProgramExecute(s.win, s.ctrlName)											// these buttons have a different execute function (no difference calculation)
		else
			DiffGen_ProcessEvents(s.win, s.ctrlName)
		endif
	endif
	return 0
End
//+++++++++++++++++++++++++++++++++++++ button help function +++++++++++++++++++++++++++++++++++++
static Function DiffGen_ProgramExecute(inPath, type)
	String inPath, type
	String gName = getGraph(inPath)																// can be called from AskQuitPanel as well => construct panel path manually
	String pPath = getPanel(gName)
	StrSwitch(type)
		case "bProgSaveQuit":
			if (WinType(gName+"#AskQuitPanel") == 7)
				KillWindow $(gName+"#AskQuitPanel")
			endif
			DiffGen_SaveAndGraphResult(pPath)													// no break here => also kill window										
			if (GetKeyState(0) == 1)															// do not quit when ctrl is pressed
				break
			endif
		case "bProgUndoQuit":
			DFREF stg = getWorkFolder(pPath)
			DFREF pkroot = getWorkFolder("")
			DFREF pkg = root:Packages:
			SetWindow $pPath hook(DiffWindow)=$""												// release hook to not trigger any events
			SetWindow $gName hook(DiffWindow)=$""
			KillWindow $gName																	// really kill the window, before deleting everything
			if (CountObjectsDFR(stg, 4) == 0)													// backwards compatibility - makes sure to not kill the root folder with other open instances
				KillDataFolder/Z stg
			endif
			Variable Objects = CountObjectsDFR(pkroot, 1)+CountObjectsDFR(pkroot, 2)+CountObjectsDFR(pkroot, 3)+CountObjectsDFR(pkroot, 4)
			if (Objects == 0 && DataFolderRefStatus(pkroot) != 0)
				KillDataFolder pkroot
			endif
			Objects = CountObjectsDFR(pkg, 1)+CountObjectsDFR(pkg, 2)+CountObjectsDFR(pkg, 3)+CountObjectsDFR(pkg, 4)
			if (Objects == 0)																	// clean up Packages folder if empty
				KillDataFolder pkg
			endif
			break
		case "bProgCancel":																		// just remove the sub-panel
			KillWindow #AskQuitPanel
			break
		case "bProgSwitch":																		// the switch button
			SwitchWaves(pPath)
			break
	EndSwitch
	return 0
End
//+++++++++++++++++++++++++++++++++++++ button help function +++++++++++++++++++++++++++++++++++++
static Function DiffGen_ProcessEvents(pPath, ctrlName)											// update modification parameters and recalculate difference
	String pPath, ctrlName
	
	DFREF stg = getWorkFolder(pPath)
	SVar/SDFR=stg locOriginal
	Wave/SDFR=stg Modifiers																		// Modifiers: [0] = horizontal shift, [1] = vertical shift, [2] = multiplier
	Wave origwave = $(locOriginal)
	
	Variable MinScaleLimit = 0.0001																// lower limit for scaling (multiplication)
	Variable Accelerate = 1, Rev = 1, newVal = 0												// acceleration modifier & reverse action modifier (to trigger the opposite effect)
	Variable HoldKey = GetKeyState(0)
	Accelerate = (HoldKey & 2^2) != 0 ? 10 : Accelerate											// when additional keys are pressed, modify the experience
	Accelerate = (HoldKey & 2^0) != 0 ? 0.5 : Accelerate										// gets faster (shift = 10x) or slower (ctrl = 1/2)
	Accelerate = ((HoldKey & 2^0) && (HoldKey & 2^2)) != 0 ? 0.1 : Accelerate					// super slow (1/10) with both shift and ctrl
	
	Strswitch (ctrlName)																		// the action for each button 
		case "bDown":
			Rev = -1
		case "bUp":
			ControlInfo/W=$pPath Offset
			Modifiers[1] += V_Value * Accelerate * Rev
			Modifiers[1] = abs(Modifiers[1]) < 10^-16 ? 0 : Modifiers[1]						// get rid of floating point errors
		break
		case "bLeft":
			Rev = -1
		case "bRight":
			ControlInfo/W=$pPath Shift
			Modifiers[0] += V_Value * Accelerate * Rev
			Modifiers[0] = abs(Modifiers[0]) < 10^-16 ? 0 : Modifiers[0]
		break
		case "bStretch":
			Rev = -1
		case "bShrink":
			ControlInfo/W=$pPath Multiplier
			Modifiers[2]  = (Modifiers[2] == MinScaleLimit && Rev == -1) ? 0 : Modifiers[2]		// scroll away smoothly from minimum limit
			Modifiers[2] -= (V_Value * Accelerate * Rev)/100									// do the modification
			Modifiers[2]  = Modifiers[2] < MinScaleLimit ? MinScaleLimit : Modifiers[2]			// make sure not to shrink the data to zero or invert
		break
		case "bUndo":																			// resets the modifications by overwriting with original data
			Modifiers = 0; Modifiers[2] = 1														// reset modify parameters
			SetVariable WaveConv	,win=$pPath ,value=_NUM:0									// reset convolution control
			SetVariable RefWaveConv	,win=$pPath ,value=_NUM:0
			if((GetKeyState(0) & 2^0) != 0)														// also reset reference wave to none if ctrl key is pressed
				PopUpMenu RefSelect ,win=$pPath ,mode=1
				DiffGen_WaveSelect(pPath, "RefSelect","none")
			endif
		break
		case "bUndoShift":
			Modifiers[0] = 0
		break
		case "bUndoOffset":
			Modifiers[1] = 0
		break
		case "bUndoScale":
			Modifiers[2] = 1
		break
		case "cPin":
			ControlInfo/W=$pPath PinBase
			CheckBox PinBase ,win=$pPath ,value=1-V_Value
			Variable yMin, csrPos
			DiffGen_FindBaseline(pPath, yMin, csrPos)
			Rev = V_Value ? 1 : -1
			Modifiers[1] += rev*yMin*(1 - Modifiers[2])											// add or subtract the scaled amount to the offset (prevents 'snapping')
		break
		case "smthDec":
			Rev = -1
		case "smthInc":
			String which = SelectString((HoldKey & 2^1),"WaveConv","RefWaveConv")				// alt key modifies reference
			ControlInfo/W=$pPath $which
			newVal = V_Value + Rev * abs(ChangePrecision(DimDelta(origwave,0),2))/4 * Accelerate
			newVal = newVal < 0 ? 0 : newVal
			SetVariable $which ,win=$pPath ,value=_NUM:newVal
		break
		case "setMode_1":
		case "setMode_2":
		case "setMode_3":
		case "setMode_4":
			newVal = str2num(ReplaceString("setMode_",ctrlName,""))
			ControlInfo/W=$pPath ArithMode
			if (abs(V_flag) == 3)
				PopUpMenu ArithMode	 ,win=$pPath ,mode=newVal
			endif
		break
	Endswitch
	
	DiffGen_CalculateDifference(pPath)
	return 0
End
//################################################################################################
static Function DiffGen_SaveAndGraphResult(pPath)
	String pPath
	DFREF stg = getWorkFolder(pPath)
	SVar/SDFR=stg locOriginal, locReference
	Wave/SDFR=stg/Z WaveO, WaveD, WaveA	
	
	ControlInfo/W=$pPath popAuxMode;	Variable auxMode = abs(V_Flag) == 3 ? V_Value-1 : kAuxMode	// control added in version 4.5
	ControlInfo/W=$pPath ArithMode;		Variable setMode = abs(V_flag) == 3 ? V_value-1 : 0
	
	String savedir	= ParseFilePath(1, locOriginal, ":", 1, 0)
	String origname	= ReplaceString("'",ParseFilePath(0, locOriginal, ":", 1, 0),"")
	String modname	= PossiblyQuoteName(origname + kModEnd)
	String diffname = PossiblyQuoteName(origname + StringFromList(setMode,kOutEnd))
	String auxname	= PossiblyQuoteName(origname + StringFromList(auxMode,kAuxEnd))
	Wave origwave	= $(locOriginal)
	Wave/Z refwave	= $(locReference)
	Wave/Z modwave	= $(savedir + modname)
	Wave/Z diffwave	= $(savedir + diffname)
	Wave/Z auxwave	= $(savedir + auxname)
	
	KillWaves/Z modwave, diffwave, auxwave														// try to remove previous versions (cleans up the folder)
	if (!EqualWaves(WaveO, origwave, 5))														// copy the modified wave only when something has changed
		Duplicate/O WaveO $(savedir + modname)
		Wave origwave = $(savedir + modname)													// for display purposes
		origname = modname																		// for legend update
	endif
	
	ControlInfo/W=$pPath cAuxSave
	Variable saveAux = V_Flag == 2 ? V_Value : kSaveAux											// added control in version 4.5
	
	WaveStats/Q WaveD
	if (V_numNans != DimSize(WaveD,0))															// only if there is more than NaNs, copy the difference
		ControlInfo/W=$pPath AltSaveOption
		if (GetKeyState(0) & 2^1 || V_Value)													// keyboard shortcut: Press alt to write the difference wave with the reference wave as name
			diffname = PossiblyQuoteName(NameOfWave(refwave) + StringFromList(setMode,kOutEnd))
			auxname	 = PossiblyQuoteName(NameOfWave(refwave) + StringFromList(auxMode,kAuxEnd))
			savedir	 = ParseFilePath(1, locReference, ":", 1, 0)								// change to ref folder
			Note WaveD, "\rReference name used"
		endif
		Duplicate/O WaveD $(savedir + diffname)
		Wave diffwave = $(savedir + diffname)
		if (saveAux)
			Duplicate/O WaveA $(savedir + auxname)
			Wave auxwave = $(savedir + auxname)
		endif
	else
		return 0
	endif
	// ++++++++++++++++++++++++++ create and update summary graphs +++++++++++++++++++++++++++++++
	ControlInfo/W=$pPath FinishGraph
	if (V_Value)
		Display/K=1/W=(200,50,700,300) origwave as "Result of " + NameOfWave(origwave)			// display the results
		AppendToGraph/C=(0,0,65280) refwave
		AppendToGraph/C=(0,0,0) diffwave
		if (saveAux)
			AppendToGraph/C=(0,40000,20000) auxwave
		endif
		ModifyGraph mirror(left)=1, mirror(bottom)=2, minor=1, standoff=0, ZisZ=1, notation(left)=1	
		Legend/C/N=ModificationInfoBox/F=0/A=RT/X=2.00/Y=2.00/J/B=1								// move legend to the right
		Legend/C/N=GraphInfoBox/F=0/A=LT/X=2.00/Y=2.00											// generate default legend
	endif
	
	String list = WinList("!DiffGenMainGraph*", ";","WIN:1")
	Variable win
	for (win = 0; win < ItemsInList(list); win += 1)											// update the modification labels of all graphs with this data set
		String currWin = StringFromList(win, list)
		Variable matchWave = Stringmatch(TraceNameList(currWin,";",1),"*"+origname+"*")
		Variable matchAnno = Stringmatch(AnnotationList(currWin),"*ModificationInfoBox*")
		if (matchWave && matchAnno)
			DiffGen_UpdateModLegend(stg, currWin)
		endif
	endfor
	return 0
End